/* copy from cis.css begins */
body {
  margin-top: 0;
  margin-left: 0;
  margin-bottom: 0;
  margin-right: 0;
  padding-top: 0;
  background-color: white;
  font-family: sans-serif;
  font-size: 11px;
  line-height: 14px;
  color: black;
}

h1 {
  color: #111144;
  font-size: 16pt;
}

h2 {
  margin-top: 0.8em; /* Added */
  color: #111144;
  font-size: 14pt;
}

h3 {
  color: #111144;
  font-size: 13pt;
}

h4 {
  color: #111144;
  font-size: 12pt;
}

a:link {
  color: #227ef6;
  font-weight: bold;
  text-decoration: none;
}

a:visited {
  color: #227ef6;
  font-weight: bold;
  text-decoration: none;
}

a:link:hover,a:visited:hover {
  color: #009E30;
}

td.leftcolumn  { padding: 0px 10px 0px 0px; }

td.rightcolumn { padding: 0px 0px 0px 10px; }
/* copy from cis.css ends */

tr.highlight td {
background-color: #6A9CFF;
}

label
{
float: left;
margin-right: 0.5em;
display: block
}

.submit input
{
margin-left: 0em;
}

input
{
color: #000000;
background: #ffffff;
border: 1px solid #000000
}

.submit input
{
color: #000;
background: #B5CFFF;
border: 2px outset #ffffff
}

fieldset
{
border: none; #1px solid #111144;
width: auto
}

legend
{
color: #111144;
background: #B5CFFF;
padding: 2px 6px;
font-size: 10pt;
}

#form_sort, #form_filter, #form_highlight, #form_search, #form_output
{
float: left;
}

#results
{
clear: both;
}

table.color_table
{
empty-cells: show;
border-width: 1px;
border-style: solid;
border-color: black;
border-collapse: collapse;
}

td.color
{
border-width: 1px;
border-style: solid;
border-color: black;
border-collapse: collapse;
padding: 5px;
}

td.alias
{
text-align: center;
}

table.stats
{
text-align: center;
font-family: Verdana, Geneva, Arial, Helvetica, sans-serif ;
font-weight: normal;
color: #fff;
background-color: #666;
border: 0px;
border-collapse: collapse;
border-spacing: 0px;
}

table.stats td
{
background-color: #def;
color: #000;
padding: 4px;
text-align: left;
border: 1px #fff solid;
}

table.stats th
{
background-color: #B5CFFF;
color: #000000;
padding: 4px;
text-align: left;
border-bottom: 2px #fff solid;
font-weight: bold;
}

div.option_box
{
background-color: #def;
margin-top: 4px;
margin-bottom: 4px;
}

div.result_box
{
margin-top: 10px;
margin-bottom: 10px;
}

:link img, :visited img
{
border: none;
}

